"""MQTT message model."""
#   Copyright 2019 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
from dataclasses import dataclass
from dataclasses import field
from typing import Optional
from typing import Union


@dataclass
class Message:
    """
    MQTT message identified by topic and payload.

    :ivar topic: Topic where the message is from or will be sent to
    :vartype topic: str
    :ivar payload: Content of the message
    :vartype payload: bytes or str or bytearray or None
    """

    topic: str
    payload: Optional[Union[str, bytes, bytearray]] = field(default=None)
