"""Available reading type measurement units."""
#   Copyright 2019 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
from enum import Enum


class ReadingTypeMeasurementUnit(Enum):
    """Enumeration of defined reading type measurement units."""

    BIT = "bit"
    PERCENT = "%"
    CO2_MOL = "mol"
    X10C = "X10℃"
    X100V = "X100V"
    X10PA = "X10Pa"
    NUMERIC = ""
    KELVIN = "K"
    CELSIUS = "℃"
    FAHRENHEIT = "°F"
    CELSIUS_X2 = "X2C"
    CELSIUS_X10 = "X10℃"
    PASCAL = "Pa"
    MILLIMETER_OF_MERCURY = "mmHg"
    INCH_OF_MERCURY = "inHg"
    BAR = "bar"
    ATMOSPHERE = "atm"
    MILLIBAR = "mb"
    PRESSURE_PERCENT = "%"
    MILLIBAR_X10 = "X10mb"
    MICROBAR = "μbar"
    KILO_PASCAL = "kPa"
    HUMIDITY_PERCENT = "%"
    HUMIDITY_PERCENT_X10 = "X10%"
    VOLT = "V"
    MILLIVOLT = "mV"
    CENTIVOLT = "cV"
    MOVEMENT = ""
    LIGHT_PERCENT = "%"
    METRES_PER_SQUARE_SECOND = "m/s²"
    GRAVITY = "g"
    GYROSCOPE = "deg/s"
    LOCATION = ""
    BEATS_PER_MINUTE = "bpm"
    BATTERY = "Wh"
    BATTERY_X1000 = "kWh"
    BREATHS_PER_MINUTE = "bpm"
    CALORIES = "cal"
    AMPERE = "A"
    MILLIAMPERE = "mA"
    WATT = "W"
    HORSEPOWER = "hp"
    MILLIWATT = "mW"
    METER = ""
    OUNCE_LIQUID_US = "oz"
    GALLON_UK = "gallon_u"
    GALLON_DRY_US = "gallon_d"
    LITRE = "L"
    OUNCE_LIQUID_UK = "oz_uk"
    MILLILITRE = "mL"
    METRE = "m"
    MILE = "mi"
    POINT = "pt"
    FOOT = "ft"
    INCH = "in"
    PARSEC = "pc"
    YARD = "yd"
    MILLIMETER = "mm"
    CENTIMETER = "cm"
    KILOMETER = "km"
    KILOGRAM = "kg"
    GALLON_LIQUID_US = "gal"
    ATOMIC_MASS = "u"
    TON_UK = "ton_uk"
    METRIC_TON = "t"
    TON_US = "ton_us"
    ELECTRON_MASS = "me"
    POUND = "lb"
    OUNCE = "oz"
    MILLIGRAM = "mg"
    GRAM = "g"
    DECIBEL = "dB"
    KNOT = "kn"
    KILOMETERS_PER_HOUR = "km/h"
    MILES_PER_HOUR = "mph"
    MACH = "Mach"
    SPEED_OF_LIGHT = "c"
    METER_PER_SECOND = "m/s"
    SECOND = "s"
    MINUTE = "min"
    HOUR = "h"
    MONTH = "month"
    DAY = "day"
    WEEK = "week"
    YEAR = "year"
    TESLA = "T"
    GAUSS = "G"
    MICRO_TESLA = "μT"
    SIEVERT = "C"
    BECQUEREL = "Bq"
    RUTHERFORD = "Rd"
    ROENTGEN = "Roentgen"
    RADIATION_DOSE_EFFECTIVE = "rem"
    CURIE = "Ci"
    NEWTON = "N"
    POUND_FORCE = "lbf"
    GRAVITY_FORCE = "grav"
    KILOGRAM_FORCE = "kgf"
    DYNE = "dyn"
    SQUARE_METRE = "m²"
    CUBIC_METRE = "m³"
    BYTE = "byte"
    GRADE = "grade"
    HECTARE = "ha"
    CUBIC_INCH = "in³"
    REVOLUTION = "rev"
    CENTIRADIAN = "centirad"
    RAD = "rd"
    COMPUTER_POINT = "pixel"
    DEGREE_ANGLE = "°"
    SECOND_ANGLE = "″"
    MINUTE_ANGLE = "′"
    SPHERE = "sphere"
    ARE = "a"
    RADIAN = "rad"
    HERTZ = "Hz"
    MEGA_HERTZ = "MHz"
    GIGA_HERTZ = "GHz"
    WEBER = "Wb"
    MAXWELL = "Mx"
    FARAD = "F"
    FARADAY = "Fd"
    OHM = "Ω"
    FRANKLIN = "Fr"
    ELECTRIC_CHARGE = "e"
    GILBERT = "Gi"
    JOULE = "J"
    ELECTRON_VOLT = "eV"
    HENRY = "H"
    SIEMENS = "S"
    LUMEN = "lm"
    CANDELA = "cd"
    LUX = "lx"
    TEXT = ""
    BOOLEAN = ""
